<?php
namespace Zeedhi\DataMining\Common\Service\DataCollector;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\Manager\Mongo\Types\Type;
use Zeedhi\DataMining\Common\Service\Mongo\Runner as MongoRunner;
use Zeedhi\DataMining\Common\Service\DataSource;
use Zeedhi\DataMining\Common\Service\Period;
use Zeedhi\DataMining\Common\Util\Environment;

class Schedule {

	const COLLECTION = 'importSchedule';

	protected $environment;
	protected $importService;
	protected $mongoRunner;
	protected $dataSourceService;
	protected $periodService;

	public function __construct(Environment $environment, MongoRunner $mongoRunner, Import $importService, DataSource $dataSourceService, Period $periodService) {
		$this->environment = $environment;
		$this->mongoRunner = $mongoRunner;
		$this->importService = $importService;
		$this->dataSourceService = $dataSourceService;
		$this->periodService = $periodService;
	}

	public function checkForNewImportProcesses($importType) {
		try {
			$processes = $this->getProcessesToImport($importType);
			$totalProcesses = count($processes);

			if($totalProcesses > 0){
				$this->log("There are ".$totalProcesses." processes to import");
				$this->startImportations($processes, $totalProcesses, $importType);
			} else {
				$this->log("No " .$importType. " processes to import");
			}
		} catch(\Exception $e) {
			echo "Error: {$e->getMessage()}.";
		}
	}

	protected function verifyImports($processes){
		$releasedProcess = [];
		foreach ($processes as $process) {
			$status = isset($process["status"]) ? $process["status"] : '';
			if($status != "pending"){
				array_push($releasedProcess, $process);
			}
		}
		return $releasedProcess;
	}
	protected function validateOnlyTeknisaImport(){
	    try{
           return \Zeedhi\Framework\DependencyInjection\InstanceManager::getInstance()->getParameter('validateOnlyTeknisaImport') ==  'true';
        }catch (\Exception $e){
            return false;
        }
    }

    protected function getProcessesToImport($type) {
		$filterCriteria = new FilterCriteria(self::COLLECTION);
		$filterCriteria->addCondition('date', FilterCriteria::LTE, new \MongoDB\BSON\UTCDateTime());
		$filterCriteria->addCondition('type', $type);
		$filterCriteria->addCondition('organizationId',FilterCriteria::NOT_IN, array(null, ''));
		if($this->validateOnlyTeknisaImport()){
            $filterCriteria->addCondition('organizationId',FilterCriteria::EQ, '20');
        }
		$processes = $this->mongoRunner->findBy($this->environment->getZmartConnection(), $filterCriteria);
		$releasedProcess = $this->verifyImports($processes);
		return $releasedProcess;
	}

	protected function startImportations($processes, $totalProcesses, $importType){
		for ($i = 0; $i < count($processes); $i++) {
			$process = $processes[$i];
			$this->log("(".($i+1)."/$totalProcesses) - Start import process to dataSource ".$process['dataSource']." at org ".$process['organizationId']);

			$this->environment->setOrganization($process['organizationId']);
			$this->removeProcessFromTheSchedule($process['_id']);
			$this->scheduleANewProcess($process);

			$persistType = isset($process['persistType']) ? $process['persistType'] : null;
			$period      = isset($process['period'])      ? $process['period']      : null;
			$processId   = isset($process['id'])		  ? $process['id']			: null;

			$this->importService->removeTempDataSourceCollections($process['organizationId'], $process['dataSource']);
			$processId = $this->importService->startProcessSync($process['organizationId'], $process['dataSource'], $period, $persistType, $importType);
		}
	}

	protected function removeProcessFromTheSchedule($processId) {
		$this->log("Remove process from queue");
		$filterCriteria = new FilterCriteria(self::COLLECTION);
		$filterCriteria->addCondition('_id', $processId);
		$this->mongoRunner->remove($this->environment->getZmartConnection(), $filterCriteria);
	}

	protected function scheduleANewProcess($process) {
		$scheduleExpression = null;

		if (isset($process['scheduleExpression'])) {
			$scheduleExpression = $process['scheduleExpression'];
		} else {
			$dataSource = $this->dataSourceService->findById($process['dataSource']);
			$scheduleExpression = isset($dataSource['scheduleExpression']) ? $dataSource['scheduleExpression'] : null;
		}

		if (!empty($scheduleExpression)) {
			unset($process['_id']);
			$this->log("Schedule new process");
			$process['date'] = $this->periodService->getNextRunDateFromExpression($scheduleExpression);
			$process['date'] = Type::getType(Type::MONGO_DATE)->convertToDatabaseValue($process['date']);

			$this->environment->getZmartConnection()->insert(self::COLLECTION, $process);
		}
	}

	protected function log($message) {
		echo "\n# - ".date('d/m/Y - H:i:s', time())." - ".$message.PHP_EOL;
	}
}